/***************************************************
* @brief LED灯驱动及相关实现函数
* @date  2024-2-4
* @author ZHF
****************************************************/
#include "stm32f10x_map.h"
#include "stm32f10x_nvic.h"
#include "led.h"

//h是bcfeg亮,01101110,j亮01110000  //01110110 00001110  L是00011100 B是11111110   2是11011010 3  11110010    4 01100110 0 11111100   1
//                                                         00111000 01111111        01011011      01001111     01100110   00111111   10000110
//小数点                                                                             11011011                             
/***************************数码管段选***************************/
u8 segTable[] = {
0x3f,0x06,0x5b,0x4f,0x66,0x6d,0x7d,0x07,0x7f,0x6f,           
0x77,0x7c,0x39,0x5e,0x79,0x71,0x3d,0x76,0x0f,0x0e,
0x75,0x38,0x37,0x54,0x5c,0x73,0x67,0x31,0x49,0x78,
0x3e,0x1c,0x7e,0x64,0x6e,0x59,0x80
};  //0-9 a-z
                /*  h， j    j    l      b    b    2    3    4     .*/
u8 segTablePortation[] = {0xdb,0xbf,0x86,0x80,0xbf, 0x86, 0xdb, 0xcf, 0xe6, 0xed, 0xfd, 0x87, 0xff, 0xef};
//                          2.   0    1.   .
/****************************************************
* 初始化二极管LED灯的引脚端口
******************************************************/
void LED_Init(void)
{
    RCC->APB2ENR |= 1 << 0;	    //使能AFIO
    RCC->APB2ENR |= 1 << 3;     //使能PORTB时钟
    RCC->APB2ENR |= 1 << 6;	    //使能PORTE时钟

    AFIO->MAPR |= 0x02000000;  	//设置PB4为I/O口,PB4复位后主功能为NJTRST
    GPIOB->CRL &= 0xFFF0F000;
    GPIOB->CRL |= 0x00030333;  	//PB0-2,4推挽输出1111
    GPIOB->ODR |= 0x00000010; 	//PB4输出高

    GPIOE->CRH &= 0X00000000;
    GPIOE->CRH |= 0X33333333;  	//PE8-15推挽输出
    GPIOE->ODR &= 0xFFFF00FF; 	//PE8-15输出低
}

/***************************************
* 流水灯选择，或数码管段选
* value:显示的数值对应的段选二进制值
****************************************/
void LedValue(u8 value)
{
    GPIOE->BRR  = 0xff << 8;
    GPIOE->BSRR = value << 8;
}

/***************************************
* 数码管显示不带小数点的数值
* 参数 w:显示的位置，即位选，左-右：0-7
*      value:要显示的数值
****************************************/
void SetLed(u8 w, u8 value)
{
    SEL0 = w % 2;
    SEL1 = w / 2 % 2;
    SEL2 = w / 4;
    LedValue(segTable[value]);
}

/***************************************
* 数码管显示带小数点的数值
* 参数 w:显示的位置，即位选，左-右：0-7
*      value:要显示的数值
****************************************/
void PortationDisplay(u8 w, u8 value)
{
    SEL0 = w % 2;
    SEL1 = w / 2 % 2;
    SEL2 = w / 4;
    LedValue(segTablePortation[value]);
}

/***************************************
* LED灯作为二进制数，显示数据
* 参数 led:显示的数值，0-255
****************************************/
void ledOn(u8 led)
{
    GPIOE->BRR  = 0xff << 8;
    GPIOE->BSRR = (led & 0xff) << 8;
}
